const invitationHelper = require('./Invitation')
const mailService = require('pubsweet-component-mail-service')
const logger = require('@pubsweet/logger')
const config = require('config')

const statuses = config.get('statuses')
module.exports = {
  addInvitation: async (collection, userId, role) => {
    collection.invitations = collection.invitations || []
    let matchingInvitation = collection.invitations.find(
      invitation =>
        invitation.userId === userId &&
        invitation.role === role &&
        invitation.hasAnswer === false,
    )
    if (matchingInvitation === undefined) {
      matchingInvitation = await invitationHelper.setupInvitation(
        userId,
        role,
        collection,
      )
    }
    matchingInvitation.timestamp = Date.now()
    await collection.save()
    return matchingInvitation
  },
  addAuthor: async (collection, user, res, url) => {
    if (collection.owners.includes(user.id)) {
      return res.status(200).json(user)
    }
    try {
      await mailService.setupAssignEmail(user.email, 'assign-author', url)

      return res.status(200).json(user)
    } catch (e) {
      logger.error(e)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  },
  addHandlingEditor: async (collection, user, invitation) => {
    collection.handlingEditor = {
      id: user.id,
      name: `${user.firstName} ${user.lastName}`,
      timestamp: invitation.timestamp,
      email: user.email,
      hasAnswer: invitation.hasAnswer,
      isAccepted: invitation.isAccepted,
    }
    collection.status = 'heInvited'
    collection.visibleStatus = statuses[collection.status].private
    await collection.save()
  },
  updateHandlingEditor: async (collection, isAccepted) => {
    collection.handlingEditor.hasAnswer = true
    collection.handlingEditor.isAccepted = isAccepted
    collection.handlingEditor.timestamp = Date.now()
    if (isAccepted) {
      collection.status = 'heAssigned'
    } else {
      collection.status = 'submitted'
    }
    collection.visibleStatus = statuses[collection.status].private
    await collection.save()
  },
}
